#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

void init(int argc, char * argv[], double * omegaH, double * omega_max, long * points, char * outputfile);
void calc(long points, double omega_max, double omegaH, double * omega, double * phi_re, double * phi_im);
void save(char * outputfile, long points, double * omega, double * phi_re, double * phi_im);
void help(char * argv[]);

int main(int argc, char * argv[]) {

    double omegaH, omega_max, *omega, *phi_re, *phi_im;
    long points;
    char * outputfile = malloc(0);

    init(argc, argv, &omegaH, &omega_max, &points, outputfile);

    omega = calloc(points, sizeof(double));
    phi_re = calloc(points, sizeof(double));
    phi_im = calloc(points, sizeof(double));

    calc(points, omega_max, omegaH, omega, phi_re, phi_im);

    save(outputfile, points, omega, phi_re, phi_im);

    free(outputfile);
    free(omega);
    free(phi_re);
    free(phi_im);

    return 0;
}

void calc(long points, double omega_max, double omegaH, double * omega, double * phi_re, double * phi_im) {

    long i;
    double nu, mu, chi, A, B, C, D, X, Y, P, Q;
    for(i = 0; i < points; i++) {
        omega[i] = omega_max*i/(points - 1);
        nu  = omega[i]/(omegaH*omegaH - omega[i]*omega[i]);
        chi = omegaH/(omegaH*omegaH - omega[i]*omega[i]);
        mu = 1 + chi;
        A = pow((nu*nu - mu*mu + mu), 2);
        B = 2*nu*nu*mu + 2*nu*nu - 2*mu*mu*mu + 4*mu*mu - 2*mu;
        C = pow((mu - 1), 2);
        D = B*B - 4*A*C;

        if (D < 0) {
            X = sqrt((2*sqrt(A*C) + B)/4/A);
            Y = sqrt((2*sqrt(A*C) - B)/4/A);
            P = sqrt((1 + X)*(1 + X) + Y*Y);
            Q = sqrt((1 - X)*(1 - X) + Y*Y);
            phi_re[i] = 180*acos(2*X/(P + Q))/M_PI;
            phi_im[i] = 180*acosh((P + Q)/2)/M_PI;
        }
        if (D >= 0 && B < 0) {
            P = Q = sqrt(1 + fabs(B + sqrt(D))/2/A);
            phi_re[i] = 90;
            phi_im[i] = 180*acosh(P)/M_PI;
        }
        if (D >= 0 && B >= 0) {
            Q = sqrt((B + sqrt(D))/2/A);
            if (Q <= 1)
                phi_re[i] = 180*acos(Q)/M_PI;
            else
                phi_re[i] = 0;

            phi_im[i] = 0;
        }
    }

    return ;
}

void save(char * outputfile, long points, double * omega, double * phi_re, double * phi_im) {

    FILE * fd = fopen(outputfile, "wt");

    long i;
    for(i = 0; i < points; i++) {
        fprintf(fd, "%g\t%g\t%g\n", omega[i], phi_re[i], phi_im[i]);
    }

    fclose(fd);

    return ;
}

void init(int argc, char * argv[], double * omegaH, double * omega_max, long * points, char * outputfile) {

    const double OmegaH_default = 0.25;
    const double omega_max_default = 3;
    const long points_default = 1000;
    const char outputfile_default[] = "./output.dat";

    long size;
    long i;
    double temp;

    // Default parameters
    *omegaH = OmegaH_default;

    *points = points_default;

    *omega_max = omega_max_default;

    size = strlen(outputfile_default) + 1;
    outputfile = realloc(outputfile, size);
    strcpy(outputfile, outputfile_default);

    // Parameters from console args
    // 
    // !!! Errors & Exceptions !!!
    //
    for(i = 1; i < argc; i++) {
        if(!strcmp(argv[i], "-h")) {
            help(argv);
            exit(EXIT_SUCCESS);
        }
        if(!strcmp(argv[i], "-OmegaH") && ((i + 1) != argc)) {
            i++;
            temp = strtod(argv[i], (char **)NULL);
            *omegaH = temp;
        }
        if(!strcmp(argv[i], "-max") && ((i + 1) != argc)) {
            i++;
            temp = strtod(argv[i], (char **)NULL);
            *omega_max = temp;
        }
        if(!strcmp(argv[i], "-points") && ((i + 1) != argc)) {
            i++;
            size = strtol(argv[i], (char **)NULL, 10);
            *points = size;
        }
        if(!strcmp(argv[i], "-output") && ((i + 1) != argc)) {
            i++;
            size = strlen(argv[i]) + 1;
            outputfile = realloc(outputfile, size);
            strcpy(outputfile, argv[i]);
        }
    }

    return ;
}

void help(char *argv[]) {

    printf("Program usage:\n");
    printf("%s [-output] [-max] [-OmegaH] [-points]\n", argv[0]);
    printf("\nType %s -h for print this help.\n", argv[0]);

    return ;
}
